import argparse
from typing import Union
import warnings

from systemrdl.node import AddrmapNode, RootNode
from peakrdl.plugins.exporter import ExporterSubcommandPlugin
from .exporter import MNRSExporter


class Exporter(ExporterSubcommandPlugin):
    """Exporter Entry point"""
    short_desc = "MNRS Exporter for SystemRDL"
    generates_output_file = True

    def add_exporter_arguments(self, arg_group: 'argparse._ActionsContainer') -> None:
        arg_group.add_argument(
            "--all",
            action="store_true",
            default=False,
            help="""generates all files from the given address map. True if no option is selected.
            """
        )
        arg_group.add_argument(
            "--addrmap",
            action="store_true",
            default=False,
            help="""generates addressmap header file.
            """
        )
        arg_group.add_argument(
            "--regfiles",
            action="store_true",
            default=False,
            help="""generates regfile headers from the address map.
            """
        )
        arg_group.add_argument(
            "--module_headers",
            action="store_true",
            default=False,
            help="""generates module header files for all regfiles.
            """
        )
        arg_group.add_argument(
            "--module_sources",
            action="store_true",
            default=False,
            help="""generates module source files for all regfiles.
            """
        )
        arg_group.add_argument(
            "--template",
            action="store",
            default="",
            help="""sets custom template. Should only be used together with the "addrmap" and
            "regfile" as generator options.
            """
        )
        arg_group.add_argument(
            "--overwrite",
            action="store_true",
            default=False,
            help="""force generation of files, independent of wether they already exist or not.
            Nonetheless, will not force the new generation of module files. 
            """
        )

    def do_export(self, top_node: Union[RootNode, AddrmapNode], options: 'argparse.Namespace') -> None:
        if not (options.all or options.addrmap or options.regfiles or options.module_headers or options.module_sources):
            options.all = True
        if options.template and options.all:
            raise ValueError(
                'Invalid options, can not use "all" and "template" at the same time')
        if options.template and options.module_headers:
            warnings.warn("""Using "module_headers" and "template" at the same time,
                            it is prefered to use "regfiles" as generator when using custom templates""")
        if options.template and options.module_sources:
            warnings.warn("""Using "module_sources" and "template" at the same time,
                            it is prefered to use "regfiles" as generator when using custom templates""")
        MNRSExporter(top_node).export(options)
