from typing import Union
import os
import argparse

from systemrdl.node import RootNode, AddrmapNode, RegfileNode, RegNode
from .generator import OutputFileGenerator


class MNRSExporter():
    """MNRSExporter class. Supplies Generators with jinja environment with helper functions and the top node.
    Generators itself are used to only fill jinja templates, iterating over the nodes themselves.
    """

    def __init__(self, node: Union[RootNode, AddrmapNode]) -> None:
        # If it is the root node, skip to top addrmap
        self.top_node = node.top if isinstance(node, RootNode) else node
        if isinstance(node, RootNode):
            self.top_node = node.top
        else:
            self.top_node = node
        self.file_generator = OutputFileGenerator()

    def export(self, options: 'argparse.Namespace') -> None:
        """Export function for MNRSExporter. Calls the OutputFileGenerator.

        Args:
            options (argparse.Namespace): argsparse options used to configure the OutputFileGenerator call.

        Raises:
            ValueError: Register instance in addressmap are not supported.
        """

        if any(isinstance(node, RegNode) for node in self.top_node.children()):
            raise ValueError(
                "At least one child node is an instance of RegNode, this is not supported")
        output_all = options.all
        output_addrmap = options.addrmap or output_all
        output_regfiles = options.regfiles or output_all
        output_module_headers = options.module_headers or output_all
        output_module_sources = options.module_sources or output_all
        template_to_use = options.template
        output_dir = options.output
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        overwrite = options.overwrite

        regfile_nodes = [node for node in self.top_node.children() if
                         isinstance(node, RegfileNode)]
        assert len(regfile_nodes), "Addrmap contains no RegFiles"

        if output_addrmap:
            template_name = template_to_use or "AddrMapHeader.jinja"
            filename = f"{output_dir}/{self.top_node.orig_type_name}.h"
            context = {"node": self.top_node}
            self.file_generator.run(
                template_name, context, filename, overwrite)

        if output_regfiles:
            template_name = template_to_use or "RegFileHeader.jinja"
            for regfile_node in regfile_nodes:
                context = {"regfile_node": regfile_node}
                if not os.path.exists(f"{output_dir}/gen"):
                    os.makedirs(f"{output_dir}/gen")
                filename = f"{output_dir}/gen/{regfile_node.orig_type_name}_regs.h"
                self.file_generator.run(
                    template_name, context, filename, overwrite)

        if output_module_headers or output_module_sources:
            for regfile_node in regfile_nodes:
                module_name = regfile_node.type_name
                context = {"regfile_node": regfile_node,
                           "module_name": module_name}
                if output_module_headers:
                    template_name = template_to_use or "ModuleHeader.jinja"
                    filename = f"{output_dir}/{module_name}.h"
                    self.file_generator.run(
                        template_name, context, filename, never_overwrite=True)
                if output_module_sources:
                    template_name = template_to_use or "ModuleSource.jinja"
                    filename = f"{output_dir}/{module_name}.cpp"
                    self.file_generator.run(
                        template_name, context, filename, never_overwrite=True)
